@@ -70,6 +70,23 @@ int make_mult(int num, int d);
  * of a brain damage.
  */
 
+static int _mcrypt_iv_is_needed( MCRYPT td, char* mode, int noiv) {
+   if (noiv!=0) return 0; /* no IV */
+
+   /* stream mode is treated differently because
+    * some stream ciphers use IVs even if the mode itself doesn't.
+    */
+   if ( strcmp(mode, "stream")==0 && mcrypt_enc_get_iv_size(td) != 0) {
+   	return 1;
+   }
+   
+   if ( mcrypt_enc_mode_has_iv(td) != 0 && mcrypt_enc_get_iv_size(td) != 0) {
+	return 1;
+   }
+
+   return 0;
+}
+
 int
 encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 {
@@ -92,7 +109,7 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
    int hash_size = mhash_get_block_size(hash_algorithm);
    size_t file_sum = 0;
 
-   word8 *ciphertext, *ciphertext_pad = NULL;
+   byte *ciphertext, *ciphertext_pad = NULL;
    FILE *TOF = NULL;
 
    td = mcrypt_module_open(algorithm, algorithms_directory, mode,
@@ -117,7 +134,7 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
       salt = _mcrypt_malloc(SALT_SIZE);	/* 20 bytes salt */
 
       /* Fill the salt with random data */
-      mcrypt_randomize( salt, SALT_SIZE, real_random_flag==TRUE?1:0);
+      mcrypt_randomize( salt, SALT_SIZE, real_random_flag);
    }
 
 
@@ -125,7 +142,7 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
     */
    IV = _secure_mcrypt_malloc(mcrypt_enc_get_iv_size(td));
    if (noiv==FALSE)
-	   mcrypt_randomize( IV, mcrypt_enc_get_iv_size(td), real_random_flag==TRUE?1:0);
+	   mcrypt_randomize( IV, mcrypt_enc_get_iv_size(td), real_random_flag);
    else
    	   memset( IV, 0, mcrypt_enc_get_iv_size(td));
 
@@ -223,8 +240,7 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
       }
    }
 
-   if ( (mcrypt_enc_mode_has_iv(td) != 0 || mcrypt_enc_get_iv_size(td) != 0) 
-   && noiv == FALSE) {
+   if (_mcrypt_iv_is_needed(td, mode, noiv) != 0) {
       if (write_iv(TOF, IV, mcrypt_enc_get_iv_size(td)) != 0) {
 	 err_crit("Error writing file\n");
 	 return -1;
@@ -258,9 +274,13 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
       return 1;
    }
 
-   if (bare_flag == FALSE)
+   if (bare_flag == FALSE) {
       tm = mhash_init(hash_algorithm);
 
+      if (tm==NULL)
+         err_quit(_("mhash initialization failed.\n"));
+
+   }
 
 /* Encryption Starts here */
    /* read the first n bytes of the file and store to ciphertext */
@@ -418,8 +438,8 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
    MCRYPT td;
    int pid, buf_block, start;
    word32 *IV = NULL;
-   word8 *ciphertext;
-   word8 *ciphertext_old, *ciphertext_p_old;
+   byte *ciphertext;
+   byte *ciphertext_old, *ciphertext_p_old;
    word32 *keyword;
    word32 *salt;
    char local_algorithm[50], *command=NULL;
@@ -538,14 +558,10 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
       return 1;
    }
 
-   if (mcrypt_enc_mode_has_iv(td) == 1 || mcrypt_enc_get_iv_size(td) != 0) {
-      if (noiv == FALSE) {
-	 IV = read_iv(FROMF, mcrypt_enc_get_iv_size(td));
-      } else {
-	 IV = _mcrypt_calloc(1, mcrypt_enc_get_iv_size(td));
-      }
+   if (_mcrypt_iv_is_needed(td, mode, noiv) != 0) {
+      IV = read_iv(FROMF, mcrypt_enc_get_iv_size(td));
    } else {
-      IV = NULL;
+      IV = _mcrypt_calloc(1, mcrypt_enc_get_iv_size(td));
    }
 
 #ifdef DEBUG
@@ -566,6 +582,8 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 
    if (bare_flag == FALSE) {
       tm = mhash_init(hash_algorithm);
+      if (tm==NULL)
+         err_quit(_("mhash initialization failed.\n"));
    }
 
    crcsize = hash_size;
