@@ -19,7 +19,7 @@
  */
 
 
-/* $Id: mcrypt.c,v 1.30 2002/05/26 10:26:18 nmav Exp $ */
+/* $Id: mcrypt.c,v 1.33 2002/06/07 18:06:48 nmav Exp $ */
 
 #ifndef DEFINES_H
 #define DEFINES_H
@@ -43,7 +43,7 @@
 #endif
 
 static char rcsid[] =
-    "$Id: mcrypt.c,v 1.30 2002/05/26 10:26:18 nmav Exp $";
+    "$Id: mcrypt.c,v 1.33 2002/06/07 18:06:48 nmav Exp $";
 
 char tmperr[128];
 unsigned int stream_flag = FALSE;
@@ -298,12 +298,7 @@ int main(int argc, char **argv)
       ein = info.ein;
    }
    if (info.real_random_flag == TRUE) {
-#ifdef HAVE_DEV_RANDOM
       real_random_flag = TRUE;
-#else
-      err_warn(_
-	       ("Warning: This system does not support real random data.\n"));
-#endif
    }
    
    force = info.force;
@@ -436,10 +431,11 @@ int main(int argc, char **argv)
 
 /* For RAND32... Called only here */
 #ifndef HAVE_DEV_RANDOM
-   err_warn(_("Warning: Using pseudo random numbers\n"));
-   srand(time(0) * getpid());
+      err_warn(_
+	       ("Warning: This system does not have a random device. "
+	       "Will use the included entropy gatherer.\n"));
 #endif
-
+   mcrypt_init_random();
 
 /* Normal startup 
  */
@@ -691,6 +687,8 @@ int main(int argc, char **argv)
       Bzero(argv[i], strlen(argv[i]));
    }
 
+   mcrypt_deinit_random();
+
    if (return_val != 0) {
       return 1;
    } else {
