@@ -36,6 +36,7 @@ extern int keysize;
 extern char* keymode;
 extern char* mode;
 extern int openpgp_z;
+extern int real_random_flag;
 /*--------------*/
 
 extern int total_bytes; /* openpgp.c */
@@ -83,7 +84,7 @@ file_to_buf( const char *file, int offset, size_t *ret_len )
     if ( statbuf.st_size == 0 )
         return NULL;
     len = statbuf.st_size;
-    fp = fopen( file, "r" );
+    fp = fopen( file, "rb" );
     if ( fp == NULL )
         return NULL;
     buf = _mcrypt_calloc( 1,  len );
@@ -485,9 +486,11 @@ plaintext_encode( const USTRING dat )
 void
 dek_free( DEK *d )
 {
-    mcrypt_generic_deinit( d->hd );
-    mcrypt_module_close( d->hd );
-    free( d );
+    if ( d ) {
+        mcrypt_generic_deinit( d->hd );
+        mcrypt_module_close( d->hd );
+        free( d );
+    }
 } /* dek_free */
 
 static int
@@ -513,7 +516,8 @@ dek_load( DEK *d, char *pass )
 {
     KEYGEN keygen_data;
     char* mode = DEFAULT_PGP_MODE;
-
+    int ret;
+    
     d->keylen = pgp_get_keysize( d->algo );
     keygen_data.hash_algorithm[0] = d->s2k.algo;
     keygen_data.count = d->s2k.count;
@@ -521,9 +525,11 @@ dek_load( DEK *d, char *pass )
     keygen_data.salt_size = 8;
 
     d->blocklen = mcrypt_module_get_algo_block_size( d->algo, NULL );
-    mhash_keygen_ext( d->s2k.mode, keygen_data, d->key, sizeof d->key,
+    ret = mhash_keygen_ext( d->s2k.mode, keygen_data, d->key, sizeof d->key,
                       pass, strlen( pass ) );
-
+    if (ret < 0) {
+    	err_quit(_("mhash_keygen_ext() failed.\n"));
+    }
     d->hd = mcrypt_module_open( d->algo, NULL, mode, NULL );
     if ( d->hd == MCRYPT_FAILED ) {
     	err_quit( _("Could not open module\n") );
@@ -539,7 +545,8 @@ dek_create( char* algo, char *pass  )
     char* mode = DEFAULT_PGP_MODE;
     keygenid mh_keymode;
     hashid mh_alg;
-
+    int ret;
+    
     d = _mcrypt_calloc( 1,  sizeof *d );
     d->algo = algo;
     d->keylen = pgp_get_keysize( algo );
@@ -568,9 +575,12 @@ dek_create( char* algo, char *pass  )
     keygen_data.salt = d->s2k.salt;
     keygen_data.salt_size = 8;
 
-    mcrypt_randomize( d->s2k.salt, 8, 0 );
-    mhash_keygen_ext( mh_keymode, keygen_data, d->key,
+    mcrypt_randomize( d->s2k.salt, 8, 0);
+    ret = mhash_keygen_ext( mh_keymode, keygen_data, d->key,
                       sizeof d->key, pass, strlen( pass ) );
+    if (ret < 0) {
+    	err_quit(_("mhash_keygen_ext() failed.\n"));
+    }
 
     d->hd = mcrypt_module_open( algo, NULL, mode, NULL );
     if ( d->hd == MCRYPT_FAILED ) {
@@ -755,10 +765,10 @@ encrypted_encode( const USTRING pt, const DEK *dek )
 
     rc = mcrypt_generic_init( dek->hd, (uchar *)&dek->key, dek->keylen, NULL );
     if ( rc < 0 ) {
-    	err_quit(_("mcrypt_generic_init() failed\n"));
+    	err_quit( _("mcrypt_generic_init() failed\n") );
     }
 
-    mcrypt_randomize( rndpref, dek->blocklen, 1 );
+    mcrypt_randomize( rndpref, dek->blocklen, real_random_flag);
     rndpref[dek->blocklen] = rndpref[dek->blocklen - 2];
     rndpref[dek->blocklen + 1] = rndpref[dek->blocklen - 1];
     _mcrypt_encrypt( dek->hd, rndpref, dek->blocklen + 2, NULL, 0 );
@@ -830,7 +840,7 @@ do_compress( int algo, int flush, uchar *inbuf, size_t insize,
         if ( zrc ) {
             if ( zs->msg )
                 fprintf( stderr, _("zlib error `%s'.\n"), zs->msg );
-            fprintf( stderr,_("compress: deflate returned %d.\n"), zrc );
+            fprintf( stderr, _("compress: deflate returned %d.\n"), zrc );
         }            
     } while ( zs->avail_out == 0 );
     
@@ -950,7 +960,7 @@ compressed_decode( const USTRING zip, USTRING *result )
     if ( zip->d[offset] != 1 && zip->d[offset] != 2 )
         return PGP_ERR_PKT;
 
-    err_info(_("Will decompress input file.\n"));
+    err_info( _("Will decompress input file.\n") );
     rc = do_uncompress( zip->d[offset], zip->d + offset + 1,
                         zip->len - offset - 1, &t );
     *result = t;
@@ -983,7 +993,7 @@ pgp_encrypt_file( const char *infile, const char *outfile, char *pass )
         fp = stdout;
     }
     else {
-        fp = fopen( outfile, "w" );
+        fp = fopen( outfile, "wb" );
         if ( fp == NULL )
             return PGP_ERR_FILE;
         buf = file_to_buf( infile, 0, &len );
@@ -1005,7 +1015,7 @@ pgp_encrypt_file( const char *infile, const char *outfile, char *pass )
     free( t );
 #ifdef HAVE_LIBZ
     if ( openpgp_z > 0 && tag == -1 && pt && len > 32 ) {
-        err_info(_("Output file will be compressed.\n"));
+        err_info( _("Output file will be compressed.\n") );
         zip = compressed_encode( pt, 1 );
         free( pt );
     }
@@ -1099,7 +1109,7 @@ pgp_decrypt_file( const char *infile, const char *outfile, char *pass )
         else
             pt = dat;
         if ( !rc && get_pkt_tag( pt->d, NULL ) == PKT_ONEPASS_SIG ) {
-            err_warn( _("Signatures can't be checked!\n"));
+            err_warn( _("Signatures can't be checked!\n") );
             goto fail;
         }
         if ( !rc ) {
@@ -1112,7 +1122,7 @@ pgp_decrypt_file( const char *infile, const char *outfile, char *pass )
         if ( !outfile )
             fp = stdout;
         else {
-            fp = fopen( outfile, "w" );
+            fp = fopen( outfile, "wb" );
             if ( fp == NULL )
                 return PGP_ERR_FILE;
         }
